<?php

//===== Error functions =============================================

function ErrorExit($msg, $action="history.back")
{
    $message = jschars($msg);
    if (!headers_sent())
    {
    DocType();?>
<html>
<head>
<meta http-equiv='Content-Type' content='text/html; charset=utf-8'>
<meta http-equiv='Expires' content='Tue, 01 Jan 1980 00:00:00 GMT'>
<meta http-equiv='Cache-Control' content='no-cache'>
<meta http-equiv='Pragma' content='no-cache'>
</head>
<body bgcolor='#EEEEEE'>
<?  }?>
</script>
<script>
<? if (!empty($message)) {?>
    alert("<?=$message?>");
<?}?>
    window.<?=$action?>();
</script>
</body>
</html>
<?  exit();
}

function ErrorClose($msg)
{
    ErrorExit($msg, "close");
}

//===== Trace constants and function ================================

define('TRACE_ARGS', 1);
define('TRACE_SQL', 2);
define('TRACE_GEN', 4);
define('TRACE_ALL', 65535);

function Trace($msg,$lvl = TRACE_GEN)
{
    global $_CT;

    $level = '';
    if (($fp = @fopen("$_SERVER[DOCUMENT_ROOT]/TraceLevel",'r')))
    {
    if (!feof($fp))
        $level = fgets($fp, 256);
    fclose($fp);
    }
    if (empty($level))
    $level = TRACE_ALL;

    if (($level & $lvl) == 0)
    return;

    if (file_exists("$_CT[LOG]/trace.log") && ($fp = @fopen("$_CT[LOG]/trace.log",'a')))
    {
    fwrite($fp,$msg."\n");
    fclose($fp);
    }
}

//===== JSON filter functions =======================================

function toJsonShell($arg)
{
    return escapeshellarg(json_encode($arg));
}

//===== SQL functions ===============================================

function sql_open()
{
    $str = $_SESSION['DBServ'] ? "host=".$_SESSION['DBServ'].' ' : '';
    $str .= "dbname=".$_SESSION['DBName'].' ';
    $str .= "user=".$_SESSION['DBUser'].' ';
    $str .= "password=".$_SESSION['DBPass'];

    if (($conn = @pg_connect($str)))
    return $conn;

    ErrorExit(gettext("ErrCannotAccessDatabase"));
}

function sql_query($req, $exitOnError = true)
{
    global  $DBConn;

    if (preg_match('/INSERT|UPDATE|DELETE/i', $req) == 1)
    log_action_sql($req);

    Trace("\treq=[$req]",TRACE_SQL);
    $res = @pg_query($DBConn, $req);

    if ($res || !$exitOnError)
    return $res;

    if (substr_count($req, "\n") > 10)
    $req = implode("\n", array_slice(explode("\n", $req), 0, 10))."\n...";
    ErrorExit("SQL error on [$req]: ".pg_last_error($DBConn));
}

function sql_query_params($req, $params, $exitOnError = true)
{
    global $DBConn;

    Trace("\treq=[$req] params=[".var_export($params, true)."]", TRACE_SQL);
    $res = @pg_query_params($DBConn, $req, $params);

    if ($res || !$exitOnError)
    return $res;

    if (substr_count($req, "\n") > 10)
    $req = implode("\n", array_slice(explode("\n", $req), 0, 10))."\n...";
    ErrorExit("SQL error on [$req]: ".pg_last_error($DBConn));
}

function sql_error()
{
    global $DBConn;

    return pg_last_error($DBConn);
}

function sql_rows($res)
{
    return $res ? pg_num_rows($res) : 0;
}

function sql_array($res, $num)
{
    return pg_fetch_array($res, $num, PGSQL_ASSOC);
}

function sql_field($res, $row, $fld)
{
    return pg_fetch_result($res, $row, $fld);
}

function sql_null($res, $row, $fld)
{
    return pg_field_is_null($res, $row, $fld);
}

function sql_type($res, $col)
{
    return pg_field_type($res, $col);
}

function sql_table($table, $handler)
{
    $res = sql_query("SELECT * FROM $table");
    $numcols = pg_num_fields($res);
    $keywords = ['user', 'default']; // Full list http://www.postgresql.org/docs/current/static/sql-keywords-appendix.html

    $sep = "";
    $cols = "";
    for ($col = 0; $col < $numcols; $col++)
    {
    $types[] = pg_field_type($res, $col);
    $name = pg_field_name($res, $col);
    if (in_array($name, $keywords))
        $filteredNames[] = '"'.$name.'"';
    else
        $filteredNames[] = $name;
    $names[] = $name;
    $cols .= $sep.$filteredNames[$col];
    $sep = ", ";
    }

    for ($i = 0; $i < pg_num_rows($res); $i++)
    {
    $sep = "";
    $vals = "";
    $row = pg_fetch_array($res, $i);
    for ($col = 0; $col < $numcols; $col++)
    {
        $val = $row[$names[$col]];
        if (preg_match("/char|text|bool/i", $types[$col]))
        {
        $quote = "'";
        $empty = "";
        $val = str_replace("\n", "\\n", str_replace("\r\n", "\\r\\n", pg_escape_string($val)));
        }
        elseif (preg_match("/date|timestamp/i", $types[$col]))
        {
        $quote = empty($val) ? "" : "'";
        $empty = "NULL";
        }
        else
        {
        $quote = "";
        $empty = "NULL";
        }
        if (empty($val) && $val != "0")
        $val = $empty;

        $vals .= $sep.$quote.$val.$quote;
        $sep = ", ";
    }
    $handler("INSERT INTO $table ($cols) VALUES($vals);");
    }
}

function srv_name($id)
{
    return $_SESSION['Srvs'][$id]['name'];
}

function cam_name($id_cam,$id_srv = 0)
{
    return value($_SESSION['Srvs'][$id_srv]['cam_names'],$id_cam,(count($_SESSION['Srvs']) > 1 ? "Server${id_srv}_" : "")."Camera$id_cam");
}

function cam_id($name)
{
    foreach ($_SESSION['Srvs'][0]['cam_names'] as $cam_id => $cam_name)
    {
        if ($cam_name == $name)
        return $cam_id;
    }
    return 0;
}

function cam_exists($id_cam)
{
    return !empty($_SESSION['Srvs'][0]['cam_names'][$id_cam]);
}

function grp_name($id)
{
    $res = sql_query("SELECT name FROM groups WHERE id_grp = $id");
    return sql_rows($res) > 0 ? sql_field($res, 0, 0) : "Group$id";
}

function usr_name($id)
{
    $res = sql_query("SELECT name FROM users WHERE id_usr = $id");
    return sql_rows($res) > 0 ? sql_field($res, 0, 0) : "User$id";
}

function prf_name($id)
{
    $res = sql_query("SELECT name FROM profiles WHERE id_prf = $id");
    return sql_rows($res) > 0 ? sql_field($res, 0, 0) : "Profile$id";
}

function usr_prof($id)
{
    $res = sql_query("SELECT p.name FROM profiles p,users u WHERE p.id_prf = u.id_prf AND id_usr = $id");
    return sql_rows($res) > 0 ? sql_field($res, 0, 0) : "PrfUsr$id";
}

function ptz_log($args)
{
    $srv = srv_name($args['id_srv']);
    $cam = cam_name($args['id_cam'], $args['id_srv']);
    $arg = $args['arg'];

    switch ($args['cmd'])
    {
    case 'ptz':
        switch ($arg)
        {
        case 'up':	log_action("LogViewCamPtzUp|$cam|$srv");	break;
        case 'down':	log_action("LogViewCamPtzDown|$cam|$srv");	break;
        case 'right':	log_action("LogViewCamPtzRight|$cam|$srv");	break;
        case 'left':	log_action("LogViewCamPtzLeft|$cam|$srv");	break;
        case 'plus':	log_action("LogViewCamPtzPlus|$cam|$srv");	break;
        case 'minus':	log_action("LogViewCamPtzMinus|$cam|$srv");	break;
        case 'out':	log_action("LogViewCamPtzOut|$cam|$srv");	break;
        }
        break;

    case 'zoom':
        if ($arg < 0)
        log_action("LogViewCamPtzMinus|$cam|$srv");
        else
        log_action("LogViewCamPtzPlus|$cam|$srv");
        break;

    case 'preset':	log_action("LogViewCamPreset|$cam|$srv|$arg");	break;
    case 'output':	log_action("LogViewCamOutput|$cam|$srv|$arg");	break;
    case 'center':	log_action("LogViewCamCenter|$cam|$srv|$arg");	break;
    case 'area':	log_action("LogViewCamArea|$cam|$srv|$arg");	break;
    }
}

//===== Themes and CSS functions ====================================

//  Returns an array of subdirectories of $dir
function SubDirs($dir)
{
    $list = array();
    if ($dh = @opendir($dir))
    {
    while (($file = readdir($dh)) !== false)
    {
        // only keep subdirectories and remove '.' and '..'
        if ($file != '.' && $file != '..' && is_dir("$dir/$file"))
        $list[] = $file;
    }
    closedir($dh);
    }
    return $list;
}

//  Returns an array of theme names
function ThemeList()
{
    $dir = "$_SERVER[DOCUMENT_ROOT]/images";
    $list = array();
    foreach(SubDirs($dir) as $sub)
    {
    if ($dh = @opendir("$dir/$sub"))
    {
        while (($file = readdir($dh)) !== false)
        {
        if (is_file("$dir/$sub/$file") && preg_match("/\.css$/", $file))
            $list[] = basename($file, ".css");
        }
        closedir($dh);
    }
    }
    sort($list);
    return $list;
}

//  Returns the (possibly internationalized) text associated with a given theme
function ThemeText($theme)
{
    return in_array($theme, array("classic","grey","purple","blue","green","orange","red")) ? gettext("Thm".ucfirst($theme)) : $theme;
}

//  Returns the color associated with a given theme
function ThemeColor($theme)
{
    $dir = "$_SERVER[DOCUMENT_ROOT]/images";
    foreach(SubDirs($dir) as $color)
    {
    if (file_exists("$dir/$color/$theme.css"))
        return $color;
    }
    return "classic";
}

//===== Send control commands to local scamd ========================

function ctl_cmd($send)
{
    $recv = '';
    if (($socket = @socket_create(AF_INET, SOCK_STREAM, getprotobyname('tcp'))))
    {
        if (@socket_connect($socket, '127.0.0.1', $_SESSION['CtrlPort'])) {
            socket_write($socket, "$send\n");
            socket_shutdown($socket, 1);
            while (($read = socket_read($socket, 4096)))
                $recv .= $read;
        }
        socket_close($socket);
    }
    else
        Trace("\tctl_cmd: ".socket_strerror(socket_last_error())." [code=".socket_last_error()."]");

    return explode("\n", rtrim($recv, "\n"));
}

function licenseInfos()
{
    global $_CT;

    $infos = [];
    $output = ctl_cmd('license');

    // Scamd is probably stopped
    if (!is_array($output) || count($output) == 0 || empty($output[0])) {
        $output = [];

        exec("$_CT[BIN]/scamd -l", $output);
    }

    if (!is_array($output) || count($output) == 0 || ($infos = json_decode($output[0], true)) === false)
        return null;

    return $infos;
}

// Opens upgrade tgz file, check expiration date and get README file text if available
function getUpgradeInfos($updateFilePath, &$readme)
{
    $readmeFilePath = './README';
    $buildFilePath = './.builddate';
    $readme = '';
    $expired = false;

    $tmpFile = tempnam(sys_get_temp_dir(), '');
    if (file_exists($tmpFile))
        unlink($tmpFile);
    if (!mkdir($tmpFile))
        return false;

    exec("tar -xf $updateFilePath -C $tmpFile $readmeFilePath $buildFilePath",$out,$ret);
    if (file_exists("$tmpFile/$readmeFilePath")) {
        $readme = file_get_contents("$tmpFile/$readmeFilePath");
        unlink("$tmpFile/$readmeFilePath");
        if ($readme === false)
            $readme = 'Error : unreadable content';
    }
    if (file_exists("$tmpFile/$buildFilePath")) {
        $builddate = file_get_contents("$tmpFile/$buildFilePath");
        unlink("$tmpFile/$buildFilePath");
        if ($builddate === false) {
            //unreadable builddate => we consider it as a bad package (expired = true in any case)
            $expired = true;
        } else {
            $license = licenseInfos();
            if (isset($license['upgrade_expiration_date'])) {
                $upgradeExpirationDate = $license['upgrade_expiration_date'];
                if ($builddate > $upgradeExpirationDate) {
                    //builddate is after upgradeExpirationDate => expired = true
                    $expired = true;
                }
            }
        }
    }
    rmdir($tmpFile);
    return $expired;
}

//===== Return selected license key data ============================

function keydata($arg)
{
    $infos = licenseInfos();

    $vars = [
        'modules' => (isset($infos['active_modules']) && is_array($infos['active_modules'])) ? $infos['active_modules'] : [],
        'max_screens' => (isset($infos['max_screens'])) ? $infos['max_screens'] : 0,
        'max_video_sources' => (isset($infos['max_video_sources'])) ? $infos['max_video_sources'] : 0,
        'max_detect_index' => (isset($infos['max_detect_index'])) ? $infos['max_detect_index'] : null,
        'serial' => (isset($infos['serial_number'])) ? $infos['serial_number'] : ''
    ];

    if (isset($vars[$arg]))
        return $vars[$arg];

    return null;
}

//===== Delete upload key from $_SESSION ============================

function upload_done($args)
{
    $pfx = ini_get("session.upload_progress.prefix");
    $key = $pfx.value($args,ini_get("session.upload_progress.name"),'');
    if (isset($_SESSION[$key]))
    {
    unset($_SESSION[$key]);
    Trace("\tupload done key=$key");
    }
}

//===== HTML functions ==============================================

define('HDR_NONE',   	0);
define('HDR_CHARSET',	1);
define('HDR_NOCACHE',	2);
define('HDR_NOSTYLE',	3);
define('HDR_STYLES', 	4);
define('HDR_ALL',   	7);
define('HDR_MOBILESTYLE',8);
define('HDR_REFRESH',	16);
define('HDR_JQUERY',	32);
define('HDR_JQUERYUI',	64);

function DocType()
{
//<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Strict//EN">
?>
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<?
}

function Entetes($what,$title = '',$main = '')
{
    global	$HostName, $_CT;

    if ($what & HDR_CHARSET)
    {?>
<meta http-equiv='Content-Type' content='text/html; charset=utf-8'>
<?  }

    if ($what & HDR_NOCACHE)
    {?>
<meta http-equiv='Expires' content='Tue, 01 Jan 1980 00:00:00 GMT'>
<meta http-equiv='Cache-Control' content='no-cache'>
<meta http-equiv='Pragma' content='no-cache'>
<?  }

    if ($what & HDR_REFRESH && $_SESSION['Config']['refresh_rate'] > 0)
    {?>
<meta http-equiv='Refresh' content='<?=$_SESSION['Config']['refresh_rate']?>'>
<?  }

    if ($title != '')
    {
    $ttl = ($title != ' ') ? "$title - " : '';
    if (isset($_SESSION['UserName']))	// Can be called before user set (login)
    {
        $ttl .= ($_SESSION['UserName'] != '') ? $_SESSION['UserName'].'@' : '';
        $ttl .= ($_SESSION['Config']['site_name'] != '') ? $_SESSION['Config']['site_name'] : "(CamTrace)";
    }?>
<title><?=$ttl?></title>
<?  }

    if ($_SESSION['WinMSIE'])
    {?>
<meta http-equiv="X-UA-Compatible" content="IE=EmulateIE9">
<?  }

    if ($what & HDR_STYLES) {?>
<link rel='stylesheet' type='text/css' href='/images/<?=$_SESSION['ThemeColor']?>/<?=$_SESSION['ThemeName']?>.css'>
<?  }

    if (($what & HDR_JQUERY) || ($what & HDR_JQUERYUI)) {?>
<script type="text/javascript" src="/lib/js/jquery.js"></script>
<?	if ($what & HDR_JQUERYUI) {?>
<script type="text/javascript" src="/lib/js/jquery-ui.js"></script>
<?	}
    }

    if ($what & HDR_MOBILESTYLE) {?>
<link rel='stylesheet' href='/images/mobiles.css' type='text/css'>
<?	if ($_SESSION['IPhone']) {?>
<link rel='stylesheet' href='/images/mobiles-iphone.css' type='text/css'>
<link rel="apple-touch-icon" href="/lib/camtrace.png"/>
<?	}
    }

//===== Error/debug system ==========================================
?>
<script>
var lang = '<?=LANG?>';
<?  if (!empty($main)) {?>
var main = <?=$main?>;
<?  }?>

<?
if (file_exists("$_CT[LOG]/trace.log"))
{
    $self = substr($_SERVER['PHP_SELF'], 1, -4); // remove .php
?>

var dbg_win = (typeof main != 'undefined' && main.dbg_win) ? main.dbg_win : null;

var Log = {
    d: function (tag, message) {
    Log._write('d', tag, message);
    },
    e: function (tag, message) {
    Log._write('e', tag, message);
    },
    _write: function (type, tag, message) {
    var now = Log._now();
    var tag = tag || "<?= $self ?>";

    if (top.console === undefined || <?=file_exists("$_SERVER[DOCUMENT_ROOT]/ForceDebugWindow") ? 'true' : 'false'?>)
    {
        if (!dbg_win || dbg_win.closed)
        {
        dbg_win = top;	// Temporarily set dbg_win to any non null value until open() returns
        dbg_win = open(location.protocol+"//"+location.host+"/debug.html","<?=$HostName?>_trace","width=780,height=400,resizable,scrollbars");
        if (typeof main != 'undefined')
            main.dbg_win = dbg_win;
        }
        try {
        dbg_win.Log[type](now, tag, message);
        } catch (error) {
        setTimeout(function () {
            Log._write(type, tag, message);
        }, 100);
        }
    }
    else
    {
        var pad = function (string, length) { for (var i = string.length; i < length; ++i) string += ' '; return string; };
        top.console.log(pad(now, 12) + ' ' + pad(tag, 22) + ' ' + message);
    }
    },
    _now: function () {
    var now = new Date(), hr, mn, se, ms;

    hr = now.getHours();
    mn = now.getMinutes();
    se = now.getSeconds();
    ms = now.getMilliseconds();
    hr = (hr < 10 ? '0' : '') + hr;	// Pads AND converts to string
    mn = (mn < 10 ? '0' : '') + mn;
    se = (se < 10 ? '0' : '') + se;
    ms = (ms < 10 ? '00' : (ms < 100 ? '0' : '')) + ms;
    return hr+":"+mn+":"+se+"."+ms;
    }
};

window.onerror = function (msg, url, line) {
    Log.e(url + ':' + line, 'JavaScript Error: ' + msg);
    return true;
}

<? } // endif (file_exists("$_CT[LOG]/trace.log")) ?>

window.debug = function (tag, message) {
<?  if (file_exists("$_CT[LOG]/trace.log") && file_exists("$_SERVER[DOCUMENT_ROOT]/Debug")) { ?>
    if (arguments.length === 1) {
    message = tag;
    tag = undefined;
    }
    Log.d(tag, message);
<?  } // else empty function ?>
}

function vdebug(embed, text)
{
<?  if (file_exists("$_CT[LOG]/trace.log") && file_exists("$_SERVER[DOCUMENT_ROOT]/Debug")) {?>
    if (typeof(embed.DrawText) != 'undefined')
    {
    embed.debugMax	      = embed.debugMax || false;
    embed.debugNextPosY   = embed.debugNextPosY || embed.ImageTop;
    embed.debugTexts      = embed.debugTexts || [];

    embed.debugTexts.push(text);
    if (embed.debugNextPosY > embed.ImageHeight)
    {
        embed.debugTexts.shift();
        for (var i = 0, h = embed.ImageTop, limit = embed.debugTexts.length / 2; i < embed.debugTexts.length; i++, h += embed.TextHeight)
        embed.DrawText(0, h, embed.debugTexts[i], -1, 20, '#ff00ff', '');
        embed.debugNextPosY = h;
    }
    else
    {
        embed.DrawText(0, embed.debugNextPosY, text, -1, 20, '#ff00ff', '');
        embed.debugNextPosY += embed.TextHeight;
    }
    }
<?  } // else empty function ?>
}

function elt_props(n,e)
{
<?  if (file_exists("$_CT[LOG]/trace.log") && file_exists("$_SERVER[DOCUMENT_ROOT]/Debug")) {?>
    for (var p in e)
    _trace(n+"["+p+"]="+e[p]);
<?  } // else empty function ?>
}
<?
//===== End of error/debug system ===================================
?>
function count(o)
{
    var n = 0;

    for (var p in o)
    n++;

    return n;
}
</script>
<?
}

function dl_nocache()
{
    if ($_SESSION['WinMSIE'] && isset($_SERVER['HTTPS']))
    {
    # Microsoft dark magic
    header('Cache-Control: public');
    header('Pragma: ');
    }
    else
    {
    header('Cache-Control: no-cache');
    header('Pragma: no-cache');
    }
}

function jschars($str)
{
    return str_replace("\n", "\\n", str_replace("\r\n","\\n",trim(htmlspecialchars($str))));
}

function htmlchars($str)
{
    return htmlspecialchars($str, ENT_QUOTES);
}

function br_text($tag)
{
    return str_replace("\n", "<br>", gettext($tag));
}

function js_text($tag)
{
    return str_replace("\n","\\n",addslashes(gettext($tag)));
}

function no_cam($id_srv, $id_cam)
{
    if (count($_SESSION['Srvs']) > 1)
        return sprintf(gettext("ErrFmtServerCameraNotFound"),$_SESSION['Srvs'][$id_srv]['name'],$id_cam);
    else
        return sprintf(gettext("ErrFmtCameraNotFound"),$id_cam);
}

function cam_dir($id)
{
    $path = null;
    $req  = "SELECT s.path";
    $req .= " FROM spaces s, cameras c";
    $req .= " WHERE c.id_spc=s.id_spc";
    $req .= "  AND c.id_cam=$1";
    $req .= " LIMIT 1";

    $res = sql_query_params($req, [$id]);
    if (sql_rows($res) === 1)
        $path = sprintf("%s/%06d", sql_field($res, 0, 'path'), $id);
    return $path;
}

function cam_ref_path($id)
{
    $path = null;
    $req  = "SELECT i.filename, s.path";
    $req .= " FROM images i, spaces s";
    $req .= " WHERE i.id_spc=s.id_spc";
    $req .= "  AND i.id_cam=$1";
    $req .= "  AND type='ref'";
    $req .= " LIMIT 1";

    $res = sql_query_params($req, [$id]);
    if (sql_rows($res) === 1)
    {
        $row = sql_array($res, 0);
        $path = "$row[path]/".sprintf("%06d", $id)."/images/$row[filename]";
    }
    return $path;
}

function cam_image_path($id, $time, $type)
{
    $path = null;
    $req  = "SELECT i.filename, s.path, s.id_spc";
    $req .= " FROM images i, spaces s";
    $req .= " WHERE i.id_spc=s.id_spc";
    $req .= "  AND i.id_cam=$1";
    $req .= "  AND i.time=$2";
    $req .= "  AND i.type=$3";
    $req .= " LIMIT 1";

    $res = sql_query_params($req, [$id, $time, $type]);
    if (sql_rows($res) === 1)
    {
        $row = sql_array($res, 0);
        $path = "$row[path]/".sprintf("%06d", $id)."/images/$row[filename]";
    }
    return $path;
}

function value($arr, $idx, $def = '')
{
    return (isset($arr[$idx]) && $arr[$idx] != '') ? $arr[$idx] : $def;
}

function boolvalue($arr, $idx, $def = '')
{
    $value = value($arr, $idx, $def);
    if ($value == 'on' || $value == 'true' || $value == 't')
    return 't';
    return 'f';
}

function reqval($idx, $def = '')
{
    if (isset($_GET[$idx]) && $_GET[$idx] != '')
    return $_GET[$idx];
    elseif (isset($_POST[$idx]) && $_POST[$idx] != '')
    return $_POST[$idx];
    return $def;
}

function reqbool($idx)
{
    if (isset($_GET[$idx]) && $_GET[$idx] != '')
    return 't';
    elseif (isset($_POST[$idx]) && $_POST[$idx] != '')
    return 't';
    return 'f';
}

function logo($t)
{
    $file = "logo_$t.png";

    return file_exists("$_SERVER[DOCUMENT_ROOT]/images/custom/$file") ? "/images/custom/$file" : "/images/$file";
}

function bigfilesize($path)
{
    if (file_exists($path))
    {
    $opts = PHP_OS == 'Linux' ? '-c "%s"' : '-f "%z"';
    exec("stat $opts ".escapeshellarg($path), $output, $ret);
    if ($ret == 0 && count($output === 1))
        return $output[0];
    return false;
    }
    return false;
}

//===== Trace page arguments ========================================

function trace_args()
{
    $proto = isset($_SERVER['HTTPS']) ? " (SSL)" : "";
    Trace(str_pad(date("Y-m-d H:i:s ").str_pad($_SERVER['REMOTE_ADDR'], 16, " ").$_SERVER['PHP_SELF']."$proto ", 78, "="),TRACE_ARGS);
    foreach ($_GET as $key => $val)
    Trace("\tGET[$key] = ".str_replace("\n", "", var_export($val,true)),TRACE_ARGS);
    foreach ($_POST as $key => $val)
    Trace("\tPOST[$key] = ".str_replace("\n", "", var_export($val,true)),TRACE_ARGS);
    foreach ($_COOKIE as $key => $val)
    Trace("\tCOOKIE[$key] = ".str_replace("\n", "", var_export($val,true)),TRACE_ARGS);
    foreach ($_FILES as $key => $val)
    {
    foreach ($val as $subkey => $subval)
        Trace("\tFILES[$key][$subkey] = $subval",TRACE_ARGS);
    }

    $ua = value($_SERVER,'HTTP_USER_AGENT','');
    $ss = value($_SERVER,'SERVER_SOFTWARE','');
    $dc = 'internal dummy connection';

    if ($ua == "$ss ($dc)")
    {
    Trace("\tDropping $dc.");
    echo "<html><body>".ucfirst($dc).".</body></html>\r\n";
    exit();
    }
}

//===== Session initialisation ======================================

function __setlang($lang, $init)
{
    global $_CT;

    putenv("LANG=$lang");
    setlocale(LC_ALL, $lang);
    setlocale(LC_NUMERIC, "C");	// So that decimal point remains '.'
    if (isset($_SERVER['CTHOME']) && $_SERVER['CTHOME'] != '' && $init)
    Trace("\tDevelopment environment in $_SERVER[CTHOME]");
    if (isset($_CT['NLS']))
    {
    if ($init)
        Trace("\tLocalized text from $_CT[NLS]");
    bindtextdomain('CamTrace', $_CT['NLS']);
    }
    textdomain('CamTrace');
}

function __setconf($vars)
{
    global $_CT;

    $lines = file("$_CT[ETC]/scamd.conf", FILE_IGNORE_NEW_LINES);
    foreach ($vars as $var => $v)
    {
    $val = '';
    if ($lines)
    {
        for ($i = 0; $i < count($lines); $i++)
        {
        if (preg_match('/^ *'.$v['tag'].' *= *([^#]*)/', $lines[$i], $matches))
        {
            $val = rtrim($matches[1]);
            break;
        }
        }
    }
    $_SESSION[$var] = empty($val) ? $v['def'] : $val;
    }
}

function __deftz()
{
    $zdir = '/usr/share/zoneinfo/';
    if (PHP_OS == 'Linux')
    {
    exec("timedatectl | awk '/Time zone:/{print $3}'", $out, $ret);
    if ($ret == 0 && count($out === 1))
        $zone = $out[0];
    }
    else if (PHP_OS == 'FreeBSD' && ($path = realpath('/etc/localtime')))
    $zone = substr($path, strlen($zdir));

    if ($zone && is_file("$zdir/$zone"))
    return $zone;

    return 'UTC';
}

//===== Main - some defines apart, top-level code begins here =======

require_once "$_SERVER[DOCUMENT_ROOT]/lib/php/environ.php";
require_once "$_SERVER[DOCUMENT_ROOT]/lib/php/ctutils.php";

$HostName = isset($_SERVER['HTTP_HOST']) ? str_replace(array("-",".",":"),"_",$_SERVER['HTTP_HOST']) : "";
$PlayPos = ",top=0,left=0";

if (session_status() == PHP_SESSION_NONE)
    session_start();

if (isset($_SESSION['UserLang']))	// Session already initialised
{
    if (isset($_SESSION['TimeZone']))
    date_default_timezone_set($_SESSION['TimeZone']);
    trace_args();	// Calls date, so at earliest here !

    if ($_SERVER['PHP_SELF'] == "/login/login.php" && (isset($_GET['lang']) || isset($_POST['lang'])))
    $_SESSION['UserLang'] = reqval('lang');
    __setlang($_SESSION['UserLang'], false);

    $DBConn = sql_open();
}
else					// Session not yet initialised
{
    if (function_exists('date_default_timezone_set'))
    date_default_timezone_set($_SESSION['TimeZone'] = __deftz());
    trace_args();	// Calls date, so at earliest here !

    Trace("\tInitialising session ".session_id());
    if (isset($_GET['lang']) || isset($_POST['lang']))
    {
    $lng = reqval('lang');
    $_SESSION['UserLang'] = strlen($lng) > 5 ? $lng : exec("$_CT[BIN]/camlang $lng");
    unset($lng);
    }
    else
    {
    $langs = "";
    foreach (explode(",", value($_SERVER,'HTTP_ACCEPT_LANGUAGE',"fr,en")) as $elt)
    {
        $arr = explode(";", $elt);
        $langs .= " $arr[0]";
    }
    $_SESSION['UserLang'] = exec("$_CT[BIN]/camlang $langs");
    unset($langs,$elt,$arr);
    }
    __setlang($_SESSION['UserLang'], true);
    __setconf(array(
    'DBServ'   => array('tag' => 'db_server', 'def' => ''),
    'DBName'   => array('tag' => 'db_name',   'def' => 'camtrace'),
    'DBUser'   => array('tag' => 'db_user',   'def' => 'camtrace'),
    'DBPass'   => array('tag' => 'db_pass',   'def' => 'camera')
    ));

    $DBConn = sql_open();

    $res = sql_query("SELECT * FROM config");
    $cnf = sql_array($res, 0);
    $_SESSION['Config'] = $cnf;

    $os = "";
    $ua = value($_SERVER,'HTTP_USER_AGENT','');
    if (stristr($ua, "mac os x"))
    $os = "MacX";
    elseif (stristr($ua, "windows"))
    $os = "WinI";
    elseif (stristr($ua, "linux"))
    $os = "LnxI";
    elseif (stristr($ua, "bsd i386"))
    $os = "BSDI";
    $_SESSION['OpSys'] = $os;
    $_SESSION['WinMSIE'] = ($os == "WinI" && (stristr($ua, "msie") || stristr($ua, "trident"))) ? true : false;
    $_SESSION['Chrome']	 = stristr($ua, 'chrome') ? true : false;
    $_SESSION['Mozilla'] = !$_SESSION['Chrome'] && stristr($ua, "gecko") ? true : false;
    $_SESSION['IPhone']  = (stristr($ua, 'iphone') || stristr($ua, 'ipad')) ? true : false;
    $_SESSION['Mobiles'] = (stristr($ua, "midp") || stristr($ua, "windows ce") || stristr($ua, " mobile ") || $_SESSION['IPhone']) ? true : false;

    $hosts = explode(':',value($_SERVER,'HTTP_HOST',''));
    $host = array_shift($hosts);

    $_SESSION['CamHost'] = $host;

    ct_init_servers();

    $_SESSION['Theme'] = $cnf['theme'];
    $_SESSION['ThemeName'] = $cnf['theme'];
    $_SESSION['ThemeColor'] = ThemeColor($_SESSION['ThemeName']);
    $_SESSION['Version'] = $cnf['db_version'];
    $_SESSION['Compress'] = $cnf['def_compress'];
    $_SESSION['BandWidth'] = $cnf['def_bandwidth'];

    $ra = $_SERVER['REMOTE_ADDR'];
    if ($ra == '127.0.0.1' && $ua == 'LocalTCP')
    {
    $_SESSION['RemoteIP'] = $ra;
    $_SESSION['isAdmin'] = true;
    $_SESSION['isMaint'] = false;
    $_SESSION['Consult'] = true;
    $_SESSION['LowBand'] = true;
    $_SESSION['Protect'] = true;
    $_SESSION['LogedIn'] = 1;
    }
    else
    $_SESSION['LogedIn'] = 0;

    unset($res,$cnf,$ua,$ra,$os,$host,$srv,$id);
}

$langs = explode('_', $_SESSION['UserLang']);
define('LANG', array_shift($langs));

//===== System global constants =====================================

define('CAL_MODELS',5);
define('CAL_PREDEF',10);
define('CAL_SLOTS', 96);

define('MAIN_MENU_W',700);
define('MAIN_MENU_H',140);

define('VIEWER_VERSION', ($_SESSION['OpSys'] == 'WinI') ? '2.0.6.0' : ($_SESSION['OpSys'] == 'MacX' ? '1.2.0.12' : '1.2.0.11'));
define('VIEWER_CLSID', "CLSID:5AA580AE-A3E9-4A9C-9C49-7EC814C8E2ED");
define('VIEWER_AFILE', "/lib/download/CamTraceViewer.cab#Version=".str_replace(".", ",", VIEWER_VERSION));
define('VIEWER_MIMETYPE', 'multipart/x-camtrace');
if ($_SESSION['OpSys'] == "MacX")
    define('VIEWER_PFILE', "/lib/download/CamTraceViewer.dmg");
else if ($_SESSION['OpSys'] != '')
    define('VIEWER_PFILE', "/lib/download/CamTraceViewer-".$_SESSION['OpSys'].".xpi");
else
    define('VIEWER_PFILE', '');

if ($_SERVER['REMOTE_ADDR'] == '127.0.0.1' && $_SESSION['OpSys'] == 'BSDI' && preg_replace('/(\d+)\.\d+-RELEASE/','$1', php_uname('r')) == '6')
    define('FLASHP_VERSION', "9.0.0.0");
else
    define('FLASHP_VERSION', "10.1.0.0");
define('FLASHP_CLSID', "CLSID:D27CDB6E-AE6D-11cf-96B8-444553540000");
define('FLASHP_AFILE', "/lib/download/FlashPlayer.cab#Version=".str_replace(".", ",", FLASHP_VERSION));
define('FLASHP_MIMETYPE', 'application/x-shockwave-flash');
if ($_SESSION['OpSys'] == "MacX")
    define('FLASHP_PFILE', "/lib/download/FlashPlayer.dmg");
elseif ($_SESSION['OpSys'] != '')
    define('FLASHP_PFILE', "/lib/download/FlashPlayer-".$_SESSION['OpSys'].".xpi");
else
    define('FLASHP_PFILE', '');

//===== Session acces controls ======================================

require_once "$_SERVER[DOCUMENT_ROOT]/lib/php/auth.php";
require_once "$_SERVER[DOCUMENT_ROOT]/lib/php/log.php";

if (isset($_SESSION['LogedIn']) && $_SESSION['LogedIn'] == 1)
{
    if ($_SESSION['Mobiles'] && $_SERVER['PHP_SELF'] == "/index.php")
    header("Location: /m");
}
else
{
    if ($_SESSION['Mobiles'] && $_SERVER['PHP_SELF'] != "/m/login.php")
    {
    header("Location: /m/login.php");
    exit();
    }

    switch ($_SERVER['PHP_SELF'])
    {
    case "/login/login.php":
    case "/login/check.php":

    case "/lib/query.php":

    case "/m/login.php":
    case "/start.php":
        break;

    // Redirect to /login/login.php if not loged in
    default:
        ?>
        <html>
        <head>
            <meta http-equiv='Content-Type' content='text/html; charset=utf-8'>
            <script type="text/javascript">
            window.onload = function () {
            var login_win = window.open("/login/login.php","<?=$HostName?>_menu","width=430,height=280,resizable=1");
            var is_chrome = (navigator.userAgent.toLowerCase().indexOf('chrome/') >= 0);

            if (is_chrome === false) // Popup detection doesn't works with Chrome. The popup window is loaded and the js code executed event if popups are blocked.
            {
                setTimeout(function () {
                if (!login_win)
                    alert("<?=js_text('MsgPopupsDisabled')?>");
                else
                {
                    window.open('', '_self', 'resizable=1');
                    window.close();
                }
                }, 500);
            }
            }
            </script>
        </head>
        <body></body>
        </html>
<?	    exit(); // Needed here because this file is always require()d
        break;
    }
}
?>
